/**
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 * 
 * Contact Information:
 * beck et al. projects GmbH
 * Theresienhoehe 13
 * 80336 Munich
 * Phone: +49-89-5442530
 * mailto: info@bea.de
 */
package de.bsvrz.ars.export;

import java.awt.Font;
import java.io.File;
import java.text.ParseException;

import javax.swing.BorderFactory;
import javax.swing.UIManager;

import de.bsvrz.ars.export.gui.ExportFrame;
import de.bsvrz.ars.export.prot.SqlProperties;
import de.bsvrz.sys.funclib.commandLineArgs.ArgumentList;
import de.bsvrz.sys.funclib.debug.Debug;
import de.bsvrz.sys.funclib.losb.DAVAppBase;

/**
 * Die Haupt-Klasse mit Mainmethode für den Daten Export. Überprüft die Parameter und startet das
 * richtige Modul für den DatenExport. Wenn nicht alle Parameter angegeben wurden, wird die GUI
 * gestartet.
 * 
 * @author beck et al. projects GmbH
 * @author Phil Schrettenbrunner
 * @version $Revision: 1.2 $ / $Date: 2008/01/22 16:55:53 $ / ($Author: yvonnes $)
 */
public class DataExport extends DAVAppBase {
    private ExportProfile exportProfile;
    private boolean cmdOverwrite = false;

    /**
     * Startet die Anwendung
     * 
     * @param args Parameter für DavConnection und Datenexport
     */
    public static void main(String[] args) {
        try {
            new DataExport(args, "Datenexport");
        } catch (Exception e) {
            System.out.println("Datenexport kann nicht gestartet werden.");
            e.printStackTrace();
            System.exit(-1);
        }
    }


    /**
     * Erzeugt eine Instanz des Datenexports. Initialisiert die Software, wertet die Parameter,
     * startet ggf. die GUI und führt den Export durch.
     * 
     * @param arguments  Kommandozeilenargumente
     * @param appName    Name, mit der sich das Programm am DaV anmeldet
     * @throws Exception Bei schweren Fehlern
     */
    public DataExport(String[] arguments, String appName) throws Exception {
        super(arguments, appName);

        // Initialisieren
        logger = Debug.getLogger();
        connectToDav();
        exportProfile = ExportProfile.SINGLETON_INSTANCE;
        exportProfile.setDavConnection(getDavCon());
        argList = new ArgumentList(arguments);
        parseArguments(argList);

        // Eingabe prüfen und GUI/CMD starten
        if (exportProfile.openGui()) {
            UIManager.put("List.font", new Font("SansSerif", Font.PLAIN, 12));
            UIManager.put("SplitPane.border", BorderFactory.createEmptyBorder(5, 5, 5, 5));
            UIManager.put("SplitPaneDivider.border", BorderFactory.createEmptyBorder());
            exportProfile.loadAllObjects();

            // Erzeuge Hauptframe
            ExportFrame exportFrame = new ExportFrame(exportProfile);
            exportFrame.setSize(450, 600);
            Positioner.center(null, exportFrame);
            exportFrame.setVisible(true);
        } else {
            // Parameter auswerten und Export direkt starten
            File f = new File(exportProfile.getOutputFile());
            if (f.exists() && !cmdOverwrite) {
                logger.error("Die Datei '" + f + "' existiert bereits.");
                System.exit(1);
            }
            if (!f.createNewFile()) {
                logger.error("Die Datei '" + exportProfile.getOutputFile() + "' kann nicht geschrieben werden.\n"
                        + "Möglicherweise haben Sie keine Schreibrechter für die Datei oder den Ordner, \n" + "in dem sie erstellt werden soll");
                System.exit(1);
            }
            ExportManager exportManager = new ExportManager(exportProfile);
            exportManager.beginExport();
            exportManager.waitUntilDone();
            disconnectFromDav();
        }
    }


    /**
     * Nimmt die Aufrufargumente entgegen und wertet die den Export betreffenden aus. Legt fest, ob
     * die GUI benötigt wird.
     * 
     * @param argList Aufrufparameter.
     *
     * @see de.bsvrz.sys.funclib.application.StandardApplication#parseArguments(ArgumentList)
     */
    public void parseArguments(ArgumentList argList) {
        exportProfile.setModus(ExportProfile.Modus.CMD);
        try {
            File f = argList.fetchArgument("-datenspez").asFile();
            if (f.exists()) exportProfile.loadDataspecificationFile(f);
            else {
                logger.error("Datei mit Datenspezifikationen (" + f + ") existiert nicht.");
                System.exit(1);
            }
        } catch (Exception e) { exportProfile.setModus(ExportProfile.Modus.GUI); }
        try {
            exportProfile.setOutputFile(argList.fetchArgument("-ausgabe").getValue());
        } catch (IllegalArgumentException e) { exportProfile.setModus(ExportProfile.Modus.GUI); }
        try {
            exportProfile.setFormat(argList.fetchArgument("-format").getValue());
        } catch (IllegalArgumentException e) { exportProfile.setModus(ExportProfile.Modus.GUI); }
        try {
            exportProfile.setFrom(argList.fetchArgument("-von").getValue());
        } catch (IllegalArgumentException e) { exportProfile.setModus(ExportProfile.Modus.GUI);
        } catch (ParseException e) {
            // wenn ein Datum angegeben wurde, aber es falsch ist, brechen wir an der
            // Kommandozeile ab, auch wenn ev die GUI hätte gestartet werden müssen
            logger.error("Die Datumsangabe -von ist nicht im Format 'TT.MM.JJJJ SS.mm.ss' oder ungültig.");
            System.exit(1);
        }
        try {
            exportProfile.setUntil(argList.fetchArgument("-bis").getValue());
        } catch (IllegalArgumentException e) { exportProfile.setModus(ExportProfile.Modus.GUI);
        } catch (ParseException e) {
            // wenn ein Datum angegeben wurde, aber es falsch ist, brechen wir an der
            // Kommandozeile ab, auch wenn ev die GUI hätte gestartet werden müssen
            logger.error("Die Datumsangabe -bis ist nicht im Format 'TT.MM.JJJJ SS.mm.ss' oder ungültig.");
            System.exit(1);
        }
        try {
            File f = argList.fetchArgument("-datentypen").asFile();
            if (!f.exists()) {
                System.out.println("Angegebene Datei für DatenTypen existiert nicht.");
                System.exit(1);
            }
            try {
                exportProfile.setSqlProperties(new SqlProperties(f));
            } catch (Exception e) {
                System.out.println("Fehler beim Verarbeiten der SQL-Properties Datei: " + e.getMessage());
                System.exit(1);
            }
        } catch (IllegalArgumentException e) { 
            exportProfile.setSqlProperties(new SqlProperties());
        } // Nicht angegeben. Das ist OK
        
        cmdOverwrite = argList.fetchArgument("-ueberschreiben=nein").booleanValue();
    }
}
