/**
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 * 
 * Contact Information:
 * beck et al. projects GmbH
 * Theresienhoehe 13
 * 80336 Munich
 * Phone: +49-89-5442530
 * mailto: info@bea.de
 */
package de.bsvrz.ars.export.prot;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.util.Properties;

/**
 * Diese Klasse liest die Propertiesdatei für de AttributTabelle ein und stellt
 * Java Methoden zur Verfügung, um auf diese zuzugreifen. Sollte nichts
 * eingelesen werden können, wird ein SQL92 Standard verwendet.
 * 
 * @author beck et al. projects GmbH
 * @author Phil Schrettenbrunner
 * @version $Revision: 1.1 $ / $Date: 2008/01/22 16:39:40 $ / ($Author: yvonnes $)
 */
public class SqlProperties {
    private static final String WERTZAHL_TYP = "wertZahl.typ";
    private static final String WERTKOMMAZAHL_TYP = "wertKommazahl.typ";
    private static final String WERTBOOL_TYP = "wertBool.typ";
    private static final String WERTBOOL_TRUE = "wertBool.true";
    private static final String WERTBOOL_FALSE = "wertBool.false";
    private static final String WERTTEXT_TYP = "wertText.typ";

    private Properties sqlProp;


    /**
     * Erzeugt ein SqlProperties Objekt, das SQL92 konform ist
     */
    public SqlProperties() {
        sqlProp = new Properties();
        sqlProp.setProperty(WERTZAHL_TYP, "decimal(20)");
        sqlProp.setProperty(WERTKOMMAZAHL_TYP, "double");
        sqlProp.setProperty(WERTBOOL_TYP, "smallint");
        sqlProp.setProperty(WERTBOOL_TRUE, "1");
        sqlProp.setProperty(WERTBOOL_FALSE, "0");
        sqlProp.setProperty(WERTTEXT_TYP, "text");
    }


    /**
     * Erzeugt ein SqlProperties Objekt, das mit den in der Datei angegebenen
     * Typen und Werten arbeitet. Es müssen alle 6 Werte angegeben werden:
     * <table summary="">
     * <tr>
     * <td>wertZahl.typ</td>
     * <td>Der Typ für die Spalte wertZahl</td>
     * </tr>
     * <tr>
     * <td> wertKommazahl.typ</td>
     * <td>Der Typ für die Spalte wertKommazahl</td>
     * </tr>
     * <tr>
     * <td>wertBool.typ</td>
     * <td>Der Typ für die Spalte wertBool</td>
     * </tr>
     * <tr>
     * <td>wertBool.true</td>
     * <td>Wert, der für true steht (z.B. 1, t, TRUE etc)</td>
     * </tr>
     * <tr>
     * <td>wertBool.false</td>
     * <td>Wert, der für false steht (z.B. 0, f, FALSE etc)</td>
     * </tr>
     * <tr>
     * <td>wertText.typ</td>
     * <td>Der Typ für die Spalte wertText</td>
     * </tr>
     * </table>
     * 
     * @param f
     *            Datei in der die Properties gespeichert sind.
     * @exception FileNotFoundException
     *                Wenn die Datei nicht gefunden werden kann
     * @exception IOException
     *                When die Datei mit den Properties nicht gelesen werden
     *                kann
     */
    public SqlProperties(File f) throws FileNotFoundException, IOException, Exception {
        sqlProp = new Properties();
        sqlProp.load(new FileInputStream(f));
        if (!ensureAllPropertiesSet()) {
            throw new Exception("SQL-Propertiesdate ungültig oder unvolständig");
        }
    }


    /**
     * Ersetzt die Marker (mit %%%PROPERTIES_NAME_IN_GROSSBUCHSTABEN%%%) durch
     * die in diesm Objekt angegebenen Wert
     * 
     * @param ddl
     *            String mit den Create Table Statements
     * @return Den String, in dem alle Vorkommnisse ersetzt worden sind.
     */
    public String parseDdl(String ddl) {
        String s = ddl.replaceAll("%%%WERTZAHL_TYP%%%", sqlProp.getProperty(WERTZAHL_TYP));
        s = s.replaceAll("%%%WERTKOMMAZAHL_TYP%%%", sqlProp.getProperty(WERTKOMMAZAHL_TYP));
        s = s.replaceAll("%%%WERTBOOL_TYP%%%", sqlProp.getProperty(WERTBOOL_TYP));
        s = s.replaceAll("%%%WERTTEXT_TYP%%%", sqlProp.getProperty(WERTTEXT_TYP));
        return s;
    }


    /**
     * Liefert den String, der für True steht
     * 
     * @return der String, der für True steht
     */
    public String getBooleanTrue() {
        return sqlProp.getProperty(WERTBOOL_TRUE);
    }


    /**
     * Liefert den String, der für False steht
     * 
     * @return String, der für False steht
     */
    public String getBooleanFalse() {
        return sqlProp.getProperty(WERTBOOL_FALSE);
    }


    /**
     * Stellt sicher, dass alle Felder mit einem nichtleeren String belegt sind.
     * 
     * @return Wahr, wenn alle 6 Felder mit einem nicht leeren String belegt
     *         sind.
     */
    private boolean ensureAllPropertiesSet() {
        if (sqlProp.getProperty(WERTZAHL_TYP, "") == "") {
            return false;
        }
        if (sqlProp.getProperty(WERTKOMMAZAHL_TYP, "") == "") {
            return false;
        }
        if (sqlProp.getProperty(WERTBOOL_TYP, "") == "") {
            return false;
        }
        if (sqlProp.getProperty(WERTBOOL_FALSE, "") == "") {
            return false;
        }
        if (sqlProp.getProperty(WERTBOOL_TRUE, "") == "") {
            return false;
        }
        if (sqlProp.getProperty(WERTTEXT_TYP, "") == "") {
            return false;
        }
        return true;
    }
}
